// socketmessage.cpp
//
// Abstract a WebSockets message
//
// (C) Copyright 2016-2019 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "socketmessage.h"

SocketMessage::SocketMessage()
{
  sock_op_code=SocketMessage::Close;
}


SocketMessage::OpCode SocketMessage::opCode() const
{
  return sock_op_code;
}


void SocketMessage::setOpCode(OpCode opcode)
{
  sock_op_code=opcode;
}


QByteArray SocketMessage::payload() const
{
  return sock_payload;
}


void SocketMessage::appendPayload(const char c)
{
  sock_payload+=c;
}


void SocketMessage::clearPayload()
{
  sock_payload.clear();
}


bool SocketMessage::isControlMessage(OpCode opcode)
{
  return opcode>=SocketMessage::Close;
}
