// glasslimits.h
//
// System-wide limits and settings for GlassCoder
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef GLASSLIMITS_H
#define GLASSLIMITS_H

#define DEFAULT_SERVER_PORT 80
#define DEFAULT_AUDIO_BITRATE 128
#define DEFAULT_AUDIO_SAMPLERATE 44100
#define DEFAULT_AUDIO_DEVICE AudioDevice::Jack
#define MAX_AUDIO_CHANNELS 2
#define RINGBUFFER_SIZE 262144
#define PROCESS_TERMINATION_TIMEOUT 30000

#endif  // GLASSLIMITS_H
